@@ -1,3 +1,5 @@
+from typing import Any, Dict, List, Union
+
 from copy import deepcopy
 from importlib import import_module
 from pathlib import Path
@@ -24,6 +26,7 @@ class Database:
     '''
     @staticmethod
     def from_config(config):
+        # type: (Dict[str, Any]) -> "Database"
         '''
         Constructs a Database instance given a valid config.
 
@@ -49,9 +52,9 @@ class Database:
             filepath = Path(path)
             filename = filepath.name
             filename, _ = os.path.splitext(filename)
-            module = import_module(filename, filepath)
+            module = import_module(filename, filepath)  # type: ignore
 
-            specs.extend(module.SPECIFICATIONS)
+            specs.extend(module.SPECIFICATIONS)  # type: ignore
 
         specs = list(set(specs))
         config['specifications'] = specs
@@ -68,6 +71,7 @@ class Database:
 
     @staticmethod
     def from_json(filepath):
+        # type: (Union[str, Path]) -> "Database"
         '''
         Constructs a Database instance from a given json file.
 
@@ -91,6 +95,7 @@ class Database:
         write_mode='copy',
         exporters={},
     ):
+        # type: (Union[str, Path], Union[str, Path], List[SpecificationBase], str, str, str, Dict[str, Any]) -> None  # noqa E501
         r'''
         Creates an instance of Database but does not populate it with data.
 
@@ -149,7 +154,7 @@ class Database:
             msg = f'{hb_root} is not a directory or does not exist.'
             raise FileNotFoundError(msg)
         if Path(hb_root).name != 'hidebound':
-            msg = f'{hb_root} directory is not named hidebound.core.'
+            msg = f'{hb_root} directory is not named hidebound.'
             raise NameError(msg)
 
         self._root = root
@@ -157,7 +162,8 @@ class Database:
         self._include_regex = include_regex
         self._exclude_regex = exclude_regex
         self._write_mode = write_mode
-        self._specifications = {x.__name__.lower(): x for x in specifications}
+        self._specifications = {x.__name__.lower(): x for x in specifications} \
+            # type: Dict[str, SpecificationBase]
         self._exporters = exporters
         self.data = None
 
@@ -165,6 +171,7 @@ class Database:
         self.__exporter_lut = None
 
     def create(self):
+        # type: () -> "Database"
         '''
         Extract valid assets as data and metadata within the hidebound
         directory.
@@ -216,6 +223,7 @@ class Database:
         return self
 
     def read(self, group_by_asset=False):
+        # type: (bool) -> "DataFrame"
         '''
         Return a DataFrame which can be easily be queried and has only cells
         with scalar values.
@@ -295,6 +303,7 @@ class Database:
         return data
 
     def update(self):
+        # type: () -> "Database"
         '''
         Recurse root directory, populate self.data with its files, locate and
         validate assets.
@@ -324,6 +333,7 @@ class Database:
         return self
 
     def delete(self):
+        # type: () -> "Database"
         '''
         Deletes hidebound/data and hidebound/metadata directories and all their
         contents.
@@ -342,11 +352,9 @@ class Database:
         return self
 
     def export(self):
+        # type: () -> None
         '''
         Exports all the files found in in hidebound root directory.
-
-        Returns:
-            Database: self.
         '''
         # TODO: Find a nicer pattern for injecting exporters.
         lut = dict(girder=GirderExporter)
@@ -364,6 +372,7 @@ class Database:
                 self.__exporter_lut[key] = exporter
 
     def search(self, query, group_by_asset=False):
+        # type: (str, bool) -> "DataFrame"
         '''
         Search data according to given SQL query.
 
