@@ -1,12 +1,12 @@
+from typing import Any, Dict, Generator, List, Union
+
 from collections import defaultdict
-from itertools import dropwhile, takewhile
 from pathlib import Path
 from pprint import pformat
-import datetime
-import humanfriendly
 import os
 import re
 
+import OpenEXR as openexr
 from schematics.exceptions import DataError, ValidationError
 
 from pandas import DataFrame
@@ -19,6 +19,7 @@ The tools module contains general functions useful to other hidebound modules.
 
 
 def list_all_files(directory, include_regex='', exclude_regex=''):
+    # type: (Union[str, Path], str, str) -> Generator[Path, None, None]
     '''
     Recusively list all files within a given directory.
 
@@ -59,6 +60,7 @@ def list_all_files(directory, include_regex='', exclude_regex=''):
 
 
 def directory_to_dataframe(directory, include_regex='', exclude_regex=r'\.DS_Store'):
+    # type: (Union[str, Path], str, str) -> DataFrame
     r'''
     Recursively list files with in a given directory as rows in a DataFrame.
 
@@ -76,7 +78,7 @@ def directory_to_dataframe(directory, include_regex='', exclude_regex=r'\.DS_Sto
         directory,
         include_regex=include_regex,
         exclude_regex=exclude_regex
-    )
+    )  # type: Any
     files = sorted(list(files))
 
     data = DataFrame()
@@ -87,58 +89,8 @@ def directory_to_dataframe(directory, include_regex='', exclude_regex=r'\.DS_Sto
     return data
 
 
-def try_(function, item, return_item='item'):
-    '''
-    Call given function on given item, catch any exceptions and return given
-    return item.
-
-    Args:
-        function (function): Function of signature lambda x: x.
-        item (object): Item used to call function.
-        return_item (object, optional): Item to be returned. Default: "item".
-
-    Returns:
-        object: Original item if return_item is "item".
-        Exception: If return_item is "error".
-        object: Object return by function call if return_item is not "item" or
-            "error".
-    '''
-    try:
-        return function(item)
-    except Exception as error:
-        if return_item == 'item':
-            return item
-        elif return_item == 'error':
-            return error
-        return return_item
-
-
-def relative_path(module, path):
-    '''
-    Resolve path given current module's file path and given suffix.
-
-    Args:
-        module (str): Always __file__ of current module.
-        path (str): Path relative to __file__.
-
-    Returns:
-        Path: Resolved Path object.
-    '''
-    module_root = Path(module).parent
-    path = Path(path).parts
-    path = list(dropwhile(lambda x: x == ".", path))
-    up = len(list(takewhile(lambda x: x == "..", path)))
-    path = Path(*path[up:])
-    root = list(module_root.parents)[up - 1]
-    output = Path(root, path).absolute()
-
-    # LOGGER.debug(
-    #     f'Relative_path called with: {module} and {path}. Returned: {output}'
-    # )
-    return output
-
-
 def error_to_string(error):
+    # type: (Exception) -> str
     '''
     Formats error as string.
 
@@ -164,6 +116,7 @@ def error_to_string(error):
 
 
 def to_prototype(dicts):
+    # type: (List[Dict]) -> Dict
     '''
     Converts a list of dicts into a dict of lists.
     .. example::
@@ -179,7 +132,7 @@ def to_prototype(dicts):
     Returns:
         dict: Prototype dictionary.
     '''
-    output = defaultdict(lambda: [])
+    output = defaultdict(lambda: [])  # type: Any
     for dict_ in dicts:
         for key, val in dict_.items():
             output[key].append(val)
@@ -187,39 +140,24 @@ def to_prototype(dicts):
     return output
 
 
-class StopWatch():
+def read_exr_header(fullpath):
+    # type: (Union[str, Path]) -> dict
     '''
-    StopWatch is used for timing blocks of code.
+    Reads an OpenEXR image file header.
+
+    Args:
+        fullpath (str or Path): Image file path.
+
+    Raises:
+        IOError: If given filepath is not an EXR file.
+
+    Returns:
+        dict: EXR header.
     '''
-    def __init__(self):
-        self._delta = None
-        self._start_time = None
-        self._stop_time = None
+    fullpath = Path(fullpath).absolute().as_posix()
+    if not openexr.isOpenExrFile(fullpath):
+        msg = f'{fullpath} is not an EXR file.'
+        raise IOError(msg)
 
-    def start(self):
-        '''
-        Call this method directly before the code you wish to time.
-        '''
-        self._stop_time = None
-        self._start_time = datetime.datetime.now()
-
-    def stop(self):
-        '''
-        Call this method directly after the code you wish to time.
-        '''
-        if self._start_time is not None:
-            self._stop_time = datetime.datetime.now()
-
-    @property
-    def delta(self):
-        '''
-        Time delta of stop - start.
-        '''
-        return self._stop_time - self._start_time
-
-    @property
-    def human_readable_delta(self):
-        '''
-        Time delta in human readable format.
-        '''
-        return humanfriendly.format_timespan(self.delta.total_seconds())
+    img = openexr.InputFile(fullpath)
+    return img.header()
