@@ -1,3 +1,20 @@
+from typing import Any, Callable, List, Union
+
+from collections import Counter
+from itertools import product
+from pathlib import Path
+import os
+import re
+
+from pandas import DataFrame
+from pyparsing import ParseException
+from schematics.exceptions import ValidationError
+import wrapt
+
+from hidebound.core.parser import AssetNameParser
+# ------------------------------------------------------------------------------
+
+
 '''
 The validators module is function library for validating singular traits given
 to a specification.
@@ -8,18 +25,9 @@ the trait they validate fails. Schematics captures these error messages and
 pipes them to an error call.
 '''
 
-import os
-import re
-
-from pyparsing import ParseException
-from schematics.exceptions import ValidationError
-import wrapt
-
-from hidebound.core.parser import AssetNameParser
-# ------------------------------------------------------------------------------
-
 
 def validate(message):
+    # type: (str) -> Callable
     '''
     A decorator for predicate functions that raises a ValidationError
     if it returns False.
@@ -44,6 +52,7 @@ def validate(message):
 
 
 def validate_each(message, list_first_arg=False):
+    # type: (str, bool) -> Callable
     '''
     A decorator for predicate functions that raises a ValidationError
     if it returns False when applied to each argument individually.
@@ -79,6 +88,7 @@ def validate_each(message, list_first_arg=False):
 # VALIDATORS--------------------------------------------------------------------
 @validate_each('"{}" is not a valid project name.')
 def is_project(item):
+    # type: (str) -> bool
     '''
     Validates a project name.
 
@@ -105,6 +115,7 @@ def is_project(item):
 
 @validate_each('"{}" is not a valid descriptor.')
 def is_descriptor(item):
+    # type: (str) -> bool
     '''
     Validates a descriptor.
 
@@ -139,11 +150,12 @@ def is_descriptor(item):
 
 @validate_each('{} is not a valid version. 0 < version < 1000.')
 def is_version(item):
+    # type: (int) -> bool
     '''
     Validates a version.
 
     Args:
-        item (str): Version.
+        item (int): Version.
 
     Raises:
         ValidationError: If version is invalid.
@@ -156,11 +168,12 @@ def is_version(item):
 
 @validate_each('{} is not a valid frame. -1 < frame < 10000.')
 def is_frame(item):
+    # type: (int) -> bool
     '''
     Validates a frame.
 
     Args:
-        item (str): Frame.
+        item (int): Frame.
 
     Raises:
         ValidationError: If frame is invalid.
@@ -176,11 +189,12 @@ def is_frame(item):
     list_first_arg=True
 )
 def is_coordinate(item):
+    # type: (List[int]) -> bool
     '''
     Validates a coordinate.
 
     Args:
-        item (str): Coordinate.
+        item (list[int]): Coordinate.
 
     Raises:
         ValidationError: If coordinate is invalid.
@@ -205,6 +219,7 @@ def is_coordinate(item):
 
 @validate_each('"{}" is not a valid extension.')
 def is_extension(item):
+    # type: (str) -> bool
     '''
     Validates a file extension.
 
@@ -224,6 +239,7 @@ def is_extension(item):
 
 @validate_each('{} != {}.')
 def is_eq(a, b):
+    # type: (Any, Any) -> bool
     '''
     Validates that a and b are equal.
 
@@ -242,6 +258,7 @@ def is_eq(a, b):
 
 @validate_each('{} !< {}.')
 def is_lt(a, b):
+    # type: (Any, Any) -> bool
     '''
     Validates that a is less than b.
 
@@ -260,6 +277,7 @@ def is_lt(a, b):
 
 @validate_each('{} !> {}.')
 def is_gt(a, b):
+    # type: (Any, Any) -> bool
     '''
     Validates that a is greater than b.
 
@@ -276,8 +294,47 @@ def is_gt(a, b):
     return a > b
 
 
+@validate_each('{} !<= {}.')
+def is_lte(a, b):
+    # type: (Any, Any) -> bool
+    '''
+    Validates that a is less than or equal to b.
+
+    Args:
+        a (object): Object.
+        b (object): Object.
+
+    Raises:
+        ValidationError: If a is not less than or equal to b.
+
+    Returns:
+        bool: A is less than or equal to b.
+    '''
+    return a <= b
+
+
+@validate_each('{} !>= {}.')
+def is_gte(a, b):
+    # type: (Any, Any) -> bool
+    '''
+    Validates that a is greater than or equal to b.
+
+    Args:
+        a (object): Object.
+        b (object): Object.
+
+    Raises:
+        ValidationError: If a is not greater than or equal to b.
+
+    Returns:
+        bool: A is greater than or equal to b.
+    '''
+    return a >= b
+
+
 @validate('{} is not homogenous.')
 def is_homogenous(items):
+    # type: (List[Any]) -> bool
     '''
     Validates thats all items are equal.
 
@@ -302,13 +359,17 @@ def is_homogenous(items):
 
 @validate_each('{} is not in {}.')
 def is_in(a, b):
+    # type: (Any, Any) -> bool
     '''
     Validates that each a is in b.
+
     Args:
         a (object): Object.
         b (object): Object.
+
     Raises:
         ValidationError: If a is not in b.
+
     Returns:
         bool: Alls a's in b.
     '''
@@ -317,13 +378,17 @@ def is_in(a, b):
 
 @validate_each('{} is not an attribute of {}.')
 def is_attribute_of(name, object):
+    # type: (str, Any) -> bool
     '''
     Validates that each name is an attribute of given object.
+
     Args:
         a (str): Attribute name.
         b (object): Object.
+
     Raises:
         ValidationError: If an name is not an attribute of given object.
+
     Returns:
         bool: Alls names are attributes of object.
     '''
@@ -332,12 +397,16 @@ def is_attribute_of(name, object):
 
 @validate('{} is not a directory or does not exist.')
 def is_directory(item):
+    # type: (Union[str, Path]) -> bool
     '''
     Validates thats item is a directory.
+
     Args:
         item (str): Directory path.
+
     Raises:
         ValidationError: If item is not a directory or does not exist.
+
     Returns:
         bool: State of item.
     '''
@@ -348,15 +417,121 @@ def is_directory(item):
 
 @validate('{} is not a file or does not exist.')
 def is_file(item):
+    # type: (Union[str, Path]) -> bool
     '''
     Validates thats item is a file.
+
     Args:
         item (str): Filepath.
+
     Raises:
         ValidationError: If item is not a file or does not exist.
+
     Returns:
         bool: State of item.
     '''
     if not os.path.isfile(item):
         return False
     return True
+
+
+def is_not_missing_values(items):
+    # type: (List[int]) -> bool
+    '''
+    Validates that sequence of integers is not missing any values.
+
+    Args:
+        items (list[int]): Integers.
+
+    Raises:
+        ValidationError: If items is missing values.
+
+    Returns:
+        bool: State of item.
+    '''
+    expected = list(range(min(items), max(items) + 1))
+    if sorted(items) == expected:
+        return True
+
+    diff = sorted(list(set(expected).difference(items)))
+    msg = f'Missing values: {diff}.'
+    raise ValidationError(msg)
+
+
+def has_uniform_coordinate_count(items):
+    # type: (List[List[int]]) -> bool
+    '''
+    Validates that non-unique list of coordinates has a uniform count per
+    coordinate.
+
+    Args:
+        items (list[list[int]]): List of coordinates.
+
+    Raises:
+        ValidationError: If coordinate count is non-uniform.
+
+    Returns:
+        bool: Uniformity of coordinates.
+    '''
+    count = Counter(list(map(str, items)))
+    if len(set(count.values())) > 1:
+        max_ = max(count.values())
+        msg = filter(lambda x: x[1] < max_, count.items())  # type: Any
+        msg = [eval(x[0]) for x in msg]
+        msg = sorted(msg)
+        msg = f'Non-uniform coordinate count. Missing coordinates: {msg}.'
+        raise ValidationError(msg)
+    return True
+
+
+def has_dense_coordinates(items):
+    # type: (List[List[int]]) -> bool
+    '''
+    Validates that list of coordinates is dense (every point is filled).
+
+    Args:
+        items (list[list[int]]): List of coordinates.
+
+    Raises:
+        ValidationError: If coordinates are not dense.
+
+    Returns:
+        bool: Density of coordinates.
+    '''
+    # build dense cartesian coordinates
+    dense = DataFrame(items) \
+        .apply(lambda x: str(list(range(x.min(), x.max() + 1)))) \
+        .tolist()
+    dense = map(eval, dense)
+    dense = map(list, product(*dense))
+    dense = list(map(str, dense))
+
+    # find difference between given coords and dense
+    coords = list(map(str, items))
+    diff = set(dense).difference(coords)  # type: Any
+    if len(diff) > 0:
+        diff = sorted(list(map(eval, diff)))
+        msg = f'Non-dense coordinates. Missing coordinates: {diff}.'
+        raise ValidationError(msg)
+    return True
+
+
+def coordinates_begin_at(items, origin):
+    # type: (List[List[int]], List[int]) -> bool
+    '''
+    Validates that the minimum coordinate of a given list equals a given origin.
+
+    Args:
+        items (list[list[int]]): List of coordinates.
+        origin (list[int]): Origin coordinate.
+
+    Raises:
+        ValidationError: If coordinates do not begin at origin.
+
+    Returns:
+        bool: State of items.
+    '''
+    if min(items) == origin:
+        return True
+    msg = f'Coordinates do not begin at {origin}.'
+    raise ValidationError(msg)
